" Project name
" let g:projName = ''
"
" Project files
" let g:projFiles = '' 


" Vim settings/maps/abbrs specific for this project

" Modeline for this file
" vim:fdm=marker:ff=unix:noet:ts=4:sw=4:ft=vim
